// Copyright 2002 Macromedia, Inc. and its licensors.  All Rights Reserved.
//
// Use and/or redistribution of this file, in whole or in part, is subject
// to the Macromedia Sample Files License, which was distributed with this file.
//
//--------------------------------------------------------------------------------
/*
 
Broadcaster class 

	The Broadcaster class implements the controller mechanism of the
	Model/View/Controller (MVC) architecture.

	Classes can either extend or encapsulate Broadcaster in order to provide
	controller services.

	View classes interact with the Broadcasters via registration and listening.
	View classes can register to listen to all Broadcaster events or specific
	events.  View classes implement a listenToBroadcaster method which is
	triggered by the Broadcaster.


	This file is a pure ActionScript object.

	#include info for this file:

			FLA:				main.fla

			Layer:				Actions : Class Definitions

			Frame:				1

			Library Symbol:		N/A (included on main timeline)

			Library Path:		

                                                        
*/
//--------------------------------------------------------------------------------

//----------------------------------------------------------------------
// Broadcaster
//----------------------------------------------------------------------
function Broadcaster() {
	this.eventlisteners = new Object();
	this.listeners = new Object();
}

Broadcaster.prototype.GUID = 0;

//----------------------------------------------------------------------
// Broadcaster message pump
//
// 	All instances of the Broadcaster, including instances of classes derived
// 	from the Broadcaster, share a single message stack and a single message
// 	pump which work together to implement event broadcasting.  
//
// 	Each time a Broadcaster instance is asked to broadcast an event, it is not
// 	immediately broadcasted.  Instead, the event is placed on the shared
// 	message stack and is kept there until the shared message pump pulls it off
// 	the stack.
//
// 	The shared message pump is implemented with an empty movie clip which
// 	contains an onEnterFrame function.  The onEnterFrame function pops a given
// 	number of messages from the shared message stack before surrending
// 	execution back to the main timeline.
//
// 	This implementation helps prevent the movie from being requested to process
// 	excessive UI events in a single frame.  This can happen if the movie
// 	receives multiple events at once and/or if events have multiple listeners.
//----------------------------------------------------------------------

Broadcaster.prototype.messagePump_mc = this.createEmptyMovieClip("pump", 99999999);
Broadcaster.prototype.messagePump_mc.msgStack_array = new Array();
Broadcaster.prototype.messagePump_mc.messagesPerFrame = 1;
Broadcaster.prototype.messagePump_mc.onEnterFrame = function() {
	var i = 0;
	while(i < this.messagesPerFrame && this.msgStack_array.length > 0) {
		var obj = this.msgStack_array.pop();
		obj.listener.listenToBroadcaster(obj.event, obj.data);
		i++;
	}
}



//----------------------------------------------------------------------
// addListener
//
// 		This method allows a View class to listen for all events for a particular
// 		Broadcaster.
//
// 		If passed a listenerID, the Broadcaster will use the value to register
// 		the listener.  If the listenerID is not unique, the existing listener
// 		with that listenerID will be replaced.
// 		
// 		If no listenerID is passed to the function, the Broadcaster will
// 		generate a globally unique listenerID for the listener and will return
// 		it from the function.
//
// 		This function returns the listenerID used to register the listener.
//----------------------------------------------------------------------
Broadcaster.prototype.addListener = function (listener, listenerID) {
	if(listenerID == undefined) listenerID = Broadcaster.prototype.GUID++;
	this.listeners[listenerID] = listener;
	return listenerID;
}

//----------------------------------------------------------------------
// addEventListener
//
// 		This method allows a View class to listen for specific events only.
// 		The specific event name is passed to this function.
//
// 		If passed a listenerID, the Broadcaster will use the value to register
// 		the listener.  If the listenerID is not unique, the existing listener
// 		with that listenerID will be replaced.
// 		
// 		If no listenerID is passed to the function, the Broadcaster will
// 		generate a globally unique listenerID for the listener and will return
// 		it from the function.
//
// 		This function returns the listenerID used to register the listener.
//----------------------------------------------------------------------
Broadcaster.prototype.addEventListener = function (event, listener, listenerID) {
	var eventListeners = this.eventlisteners[event];
	if(listenerID == undefined) listenerID = Broadcaster.prototype.GUID++;
	if(eventListeners == undefined) {
		eventListeners = new Object();
		this.eventlisteners[event] = eventListeners;
	}
	eventListeners[listenerID] = listener;
	return listenerID;
}

//----------------------------------------------------------------------
// removeListener
//
// 	Removes a listener given the listenerID returned from a previous
// 	addListener call.
//----------------------------------------------------------------------
Broadcaster.prototype.removeListener = function (listenerID) {
	this.listeners[listenerID] = undefined;
	delete this.listeners[listenerID];
}


//----------------------------------------------------------------------
// removeEventListener
//
// 	Removes a listener given the listenerID returned from a previous
// 	addEventListener call.
//----------------------------------------------------------------------
Broadcaster.prototype.removeEventListener = function (event, listenerID) {
	var eventListeners = this.eventlisteners[event];
	if(listenerID == undefined) listenerID = listener.toString();
	if(eventListeners != undefined) {
		eventListeners[listenerID] = undefined;
		delete eventListeners[listenerID];
	}
}

//----------------------------------------------------------------------
// broadcast
//
// This function is called by classes that extend or encapsulate the Broadcast
// class.  It causes listenToBroadcaster to be called on all the appropriate
// listeners and event listeners.
//----------------------------------------------------------------------
Broadcaster.prototype.broadcast = function (event, data) {
	var listener;
	var eventListeners = this.eventlisteners[event];
	if(eventListeners != undefined) {
		for(listener in eventListeners) {
			var obj = new Object();
			obj.listener = eventListeners[listener];
			obj.event = event;
			obj.data = data;
			Broadcaster.prototype.messagePump_mc.msgStack_array.push(obj);
			//eventListeners[listener].listenToBroadcaster(event, data);
		}
	}
	for(listener in this.listeners) {
		var obj = new Object();
		obj.listener = this.listeners[listener];
		obj.event = event;
		obj.data = data;
		Broadcaster.prototype.messagePump_mc.msgStack_array.push(obj);
		//this.listeners[listener].listenToBroadcaster(event, data);
	}
}

//Authors: Brian Kruse (bkruse), Buck Bito (bbito)
